<?php

namespace Buckhill\U2F\tests\units\Enroll;

require('vendor/autoload.php');

use mageekguy\atoum;
use function \Buckhill\U2F\u2f;

class Enrollment extends atoum
{
    public function testHelper()
    {
        $this
            ->array($challenge = (u2f()->enroll()->challenge('https://app.id')))->isNotEmpty();
    }
}