<?php

namespace Buckhill\U2F\Authenticate;

use Buckhill\U2F\U2FKey;

class Request
{
    protected $key;

    public function challenge(U2FKey $key)
    {
        return $key->createChallenge();
    }
}